/**********************************************************************************
* Arduino Library for the display module families
* TFT2428T, TFT50D and TFT60D from XV-Display.de
*
* This is a demonstration how to use the provided
* functions of the library. 
*
* Revision 1
*
* Note: Some functions accesses the user flash memory
* and the micro sd card. First program flash.xvc into user
* flash memory and insert a micro sd card (FAT32) with example
* pictures. The package file flash.xvc (images and fonts)
* and example windows bitmap pictures are provided in this
* demo (see folder resources).
*
* Programming user flash memory:
* (1) insert micro sd card, formattet with FAT32 and flash.xvc
*     saved on it.
* (2) power on display module. programming process starts and
*     the status will be displayed.
* (3) if programming is completed, power off and remove micro
*     sd card.
*
* Recommended development environment:
* Arduino environment
* ---> http://arduino.cc/en/Main/Software
* E.g. one of following Boards:
* Arduino Mega 2560 R3
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/Arduino-Mega-2560.html
* Arduino Uno Rev3
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/Arduino-Uno-Rev3.html
* ITEADUINO MEGA 2560
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/ITEADUINO-MEGA-2560.html
* 
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
* 
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
* 
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* Arduino boards
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
* 
* 
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file. 
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/

#include <Config.h>
#include <XVDisplay_Library.h>

#include <avr/pgmspace.h>

// Declare an instance of xvdisplay class
xvdisplay display;

// Prototypes of demo functions
void Demo_Touchpanel(void);
void Demo_WelcomeScreen(void);
void Demo_DisplayMode(void);
void Demo_SetColor(void);
void Demo_Graphic_FillScreen(void);
void Demo_Graphic_DrawRectangle(void);
void Demo_Graphic_DrawLine(void);
void Demo_Graphic_DrawPixel(void);
void Demo_Graphic_DrawCircle(void);
void Demo_DisplayText(void);
void Demo_DisplayNumbers(void);
void Demo_UserFlashMemory_Image(void);
void Demo_MicroSDCard_Image(void);
void Demo_EmbeddedFont(void);
void Demo_UserFlashMemory_Font(void);    
void Demo_Touchpanel(void);
void Demo_DrawImageData(void);
void Demo_SoftwareResetDisplayModule(void);
    
// Prototypes of helper functions for this demo
void Display_Title(const char *s);
void CreateFileName(unsigned char *s, uint8_t index);


void setup()
{
}

void loop()
{
   
  // wait 300ms or poll BUSY flag. After power up, display module
  // initializes.
  delay(300);
  while (DISPLAY_IS_BUSY) {  }
  // Display module is now ready to receive commands.

  // fading in backlight
  uint8_t i = 0;
  do {
    i++;
    display.SetBacklightIntensity(i);
    delay(10); // we need some delay to achieve effect
  } while (i < 255);

  // or just set backlight to a specific level
  display.SetBacklightIntensity(255);  // highest brightness


  while (true)
  {
    
    // Display a welcome screen
    Demo_WelcomeScreen();
    
    // Set display mode portrait / landscape
    Demo_DisplayMode();
    
    // Set pen and brush color
    Demo_SetColor();
    	
    // Graphic functions
    Demo_Graphic_FillScreen();
    Demo_Graphic_DrawRectangle();
    Demo_Graphic_DrawLine();
    Demo_Graphic_DrawPixel();
    Demo_Graphic_DrawCircle();
                
    // Display text
    Demo_DisplayText();
    
    // Display numbers
    Demo_DisplayNumbers();
    
    // Display some images stored in user flash memory
    Demo_UserFlashMemory_Image();
    
    // Display some images stored on micro sd card
    Demo_MicroSDCard_Image();
    
    // Make use of embedded font
    Demo_EmbeddedFont();
    
    // Make use of font stored in user flash memory 
    Demo_UserFlashMemory_Font();
        
    // Demonstration using touchpanel functions
    Demo_Touchpanel();
    
    // Display raw rgb data
    Demo_DrawImageData();

    // Reset display module (You do not have to software reset the
    // display module afer system start! A software reset sets system
    // parameters to initialized state)
    // Demo_SoftwareResetDisplayModule();
    
    // repeat the instruction set demo
    display.FillScreen_BrushColor();
    display.DisplayText_F(5, 5, PSTR("Repeat demo ..."));
    delay(3000);
    display.FillScreen_BrushColor();
  }


}

void Demo_WelcomeScreen(void)
{
  /********************************************************
  
  Welcome screen

  ********************************************************/

  display.DisplayImagePackageImage((DISPLAY_MODULE_WIDTH-208)/2, 100, 0, 0); // center horiz XV-DiSPLAY logo 208x36

  delay(1000);
  
  // Display some text
  display.DisplayText_F(35, 160, PSTR("This example relates to"));
  display.DisplayText_F(35, 172, PSTR("display module families"));
  
  display.DisplayText_F(65, 200, PSTR("XV-TFT2428T"));
  display.DisplayText_F(110, 210, PSTR("+"));  
  display.DisplayText_F(75, 220, PSTR("XV-TFT50D"));  
  display.DisplayText_F(110, 230, PSTR("+"));    
  display.DisplayText_F(75, 240, PSTR("XV-TFT60D"));

  delay(5000);
	
}

  /********************************************************
  Section:
    Display Mode
  
  Related functions:
    uint8_t SetDisplayMode(uint8_t mode);
    parameter: dmPortrait (display always starts in this mode)
               dmLandscape
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/
  
void Demo_DisplayMode(void)
{
	
  display.DisplayImagePackageImage(0, 0, 0, 1);  // Package0, Item0: Image 240x320  
  display.DisplayText_F(5, 5, PSTR("Display Portrait Mode"));  

  delay(4000);
  
  // Set display mode to landscape mode
  display.SetDisplayMode(dmLandscape);

  display.DisplayImagePackageImage(0, 0, 0, 2); // Package0, Item2: Image 320x240  
  display.DisplayText_F(5, 5, PSTR("Display Landscape Mode")); 
  
  delay(4000);  
  
  // Set display mode to portrait mode
  display.SetDisplayMode(dmPortrait);
  	
}

  /********************************************************
  Section:
    Set Color
  
  Related functions:
    uint8_t SetPenColor(uint8_t r, uint8_t g, uint8_t b);
    uint8_t SetBrushColor(uint8_t r, uint8_t g, uint8_t b);

    uint8_t SetPenColor_Index(uint32_t color);  // 00 bb gg rr
    uint8_t SetBrushColor_Index(uint32_t color); // 00 bb gg rr		
    
    
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/
  
void Demo_SetColor(void)
{	
  uint8_t r, g, b;
  uint16_t i;
  
  Display_Title(PSTR("Set Color"));
  delay(3000);
  r=0; g=0; b=0;
  for (i=0; i<1000; i++)
  {
  	 
  	 b += 2;
  	 g++;
  	 r += 5;
  	 
     display.SetPenColor(r, g, b);
     display.DisplayText_F(5, 35, PSTR("Set Pen Color"));
  }
  
  display.SetPenColor_Index(clBlack); // Set pen color by index

  r=0; g=0; b=0;
  for (i=0; i<1000; i++)
  {
  	 
  	 b += 2;
  	 g++;
  	 r += 5;
  	 
     display.SetBrushColor(r, g, b);
     display.DisplayText_F(5, 35, PSTR("Set Brush Color"));
  }

  display.SetBrushColor_Index(clWhite); // Set brush color by index
  delay(1000);
}


  /********************************************************
  Section:
    Graphic Fill Screen
  
  Related functions:
    uint8_t FillScreen_BrushColor(void);
    uint8_t FillScreen_PenColor(void);
    uint8_t FillScreen_RGBColor(uint8_t r, uint8_t g, uint8_t b);
    
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/

void Demo_Graphic_FillScreen(void)
{

  // FillScreen
  Display_Title(PSTR("Fill Screen"));
  delay(3000);

  display.SetPenColor_Index(clTeal); // set by color index or
  // display.SetPenColor(0, 80, 80); // set by r g b
  
  display.SetBrushColor_Index(clWhite);	 // set by color index or
 // display.SetBrushColor(255, 255, 255);// set by r g b	

  display.FillScreen_PenColor();
  delay(500);

  display.FillScreen_BrushColor();	
  delay(500);

  display.FillScreen_RGBColor(20, 40, 255);	
  delay(250);

  display.SetPenColor_Index(clBlack);	
	
}

  /********************************************************
  Section:
    Graphic Draw Rectangle
  
  Related functions:
    uint8_t DrawRect_PenColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
    uint8_t DrawRect_PenColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
    uint8_t DrawRect_BrushColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
    uint8_t DrawRect_BrushColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
    uint8_t DrawRect_RGBColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b);
    uint8_t DrawRect_RGBColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b);
        
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/

void Demo_Graphic_DrawRectangle(void)
{
  uint8_t r, g, b;
  uint16_t x;

  Display_Title(PSTR("Draw Rectangle"));
  delay(500);

  display.DrawRect_PenColor(20, 50, 220, 100);
  delay(1000);

  display.DrawRect_BrushColor(20, 50, 220, 100);
  delay(1000);

  display.DrawRect_RGBColor(20, 50, 220, 100, 20, 40, 255);
  delay(1000);
  
  display.DrawRect_RGBColorFill(20, 50, 220, 100, 20, 40, 255);	
  delay(1000);

  display.DrawRect_PenColorFill(20, 50, 220, 100);	
  delay(1000);

  display.DrawRect_BrushColorFill(20, 50, 220, 100);
  delay(1000);

  r=0; g=0; b=0;
  for (x=0; x<500; x++)
  {
    r -= 1;  // color changing 
    g += 1;
    b += 5; 
    display.DrawRect_RGBColor(20, 50, 220, 100, r, g, b);	
	delay(5);
  }
  
  for (x=0; x<500; x++)
  {
    r -= 1;  // color changing 
    g += 1;
    b += 5; 
    display.DrawRect_RGBColorFill(20, 50, 220, 100, r, g, b);	
  }
  delay(1000);

}

  /********************************************************
  Section:
    Graphic Draw Line
  
  Related functions:
    uint8_t MoveTo(uint16_t x, uint16_t y);
    uint8_t LineTo_PenColor(uint16_t x, uint16_t y);
    uint8_t LineTo_BrushColor(uint16_t x, uint16_t y);
    uint8_t LineTo_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b);
        
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/
  
void Demo_Graphic_DrawLine(void)
{
  uint8_t r, g, b, i;
  uint16_t xa, ya, xb, yb, x, y, u;
  
  Display_Title(PSTR("Draw Line"));
  delay(1500);

  display.DrawLine_PenColor(20, 50, 220, 100);	
  delay(1000);

  display.DrawLine_BrushColor(20, 50, 220, 100);	
  delay(1000);
 

  xa = 20;
  ya = 20;	 
  xb = 220;
  r=0; g=0; b=0;   
  for (i=0; i<250; i+=5)
  {
    for (yb=20; yb<300; yb+=15)
    {

      r -= 1;  // changing color
      g += 1;
      b += 5;

      display.DrawLine_RGBColor(xa, ya, xb, yb, r, g, b);	
    }
  }

  delay(1000);
  
  // MoveTo LineTo
  
  Display_Title(PSTR("MoveTo and LineTo"));
  delay(1500);

  display.MoveTo(5, 20);
  
  y=20;
  for (u=0; u<300; u+=20)
  {
	  
    for (x=10; x<230; x+=15)
    {
      if ((20+u) == y)
	    y = 30+u; 
      else
	    y = 20+u;
      display.LineTo_RGBColor(x, y, 40, 40, 255);
    }
  
  } 
  delay(2000);  
  
}

  /********************************************************
  Section:
    Graphic Draw Pixel
  
  Related functions:
    uint8_t DrawPixel_PenColor(uint16_t x, uint16_t y);
    uint8_t DrawPixel_BrushColor(uint16_t x, uint16_t y);
    uint8_t DrawPixel_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b);
        
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/
  
void Demo_Graphic_DrawPixel(void)
{
  uint8_t r, g, b;
  uint16_t x, y;
	
  Display_Title(PSTR("Draw Pixel"));
  delay(500);

  r = 20;
  g = 40;
  b = 255;

  for (x=50; x<190; x+=3)
  {
    for (y=40; y<280; y+=3)
	{
      r -= 1;  // changing color
	  g += 1;
	  b += 5; 
     
      display.DrawPixel_RGBColor(x, y, r, g, b);	
    }
  }
  delay(250);

  // for the sake of completeness ...
  display.DrawPixel_PenColor(120, 50);	 // Draw pixel with pen color
  delay(250);

  display.DrawPixel_BrushColor(120, 50); // Draw pixel with brush color	
  delay(250);

}

  /********************************************************
  Section:
    Graphic Draw Circle
  
  Related functions:    
    uint8_t DrawCircle_PenColor(uint16_t x, uint16_t y, uint8_t radius);
    uint8_t DrawCircle_BrushColor(uint16_t x, uint16_t y, uint8_t radius);
    uint8_t DrawCircle_RGBColor(uint16_t x, uint16_t y, uint8_t radius, uint8_t r, uint8_t g, uint8_t b);
        
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/
  
void Demo_Graphic_DrawCircle(void)
{
  uint8_t r, g, b, u;
  uint16_t x, y;
  
  Display_Title(PSTR("Draw Circle"));
  delay(1500);

  x = 120;
  y = 160;
  r = 20;
  g = 40;
  b = 255;

  for (x=0; x<200; x++)
  {
    r -= 1;  // changing color
	g += 1;
	b += 5;
    display.DrawCircle_RGBColor(x, y, 100, r, g, b);	
  }
  delay(500);
  
  display.DrawCircle_PenColor(120, 160, 80);	
  delay(500);

  display.DrawCircle_BrushColor(120, 160, 80);	
  delay(200);

  x = 10;
  y = 40; 
  for (u=20; u<250; u+=5)
  {
    x += 3;
	y += 5;
    display.DrawCircle_RGBColor(x, y, u, 20, 40, 255);	
  }

  delay(2000);
}


  /********************************************************
  Section:
    Display Text
  
  Related functions:
    uint8_t DisplayText(uint16_t x, uint16_t y, unsigned char *s);
    uint8_t DisplayText_F(uint16_t x, uint16_t y, const char *s);
    
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/

void Demo_DisplayText(void)
{

    Display_Title(PSTR("Display Text"));

   // Display text stored in flash memory
   display.DisplayText_F(5, 35, PSTR("Display text stored in flash"));
   display.DisplayText_F(5, 45, PSTR("memory of microcontroller"));
   
   // Display text stored in sram
   display.DisplayText(5, 65, (unsigned char *)"Display text stored in");
   display.DisplayText(5, 75, (unsigned char *)"sram of microcontroller");
   
   // SetTextMarginX defines the right x margin until strings will be
   // displayed. Command DisplayMode() sets margin to init value 0 (= margin disabled)
   // Example:
   // display.SetTextMarginX(180); 
   
   
   delay(7000);
}

  /********************************************************
  Section:
    Display Numbers
  
  Related functions:
  
    uint8_t DisplayDecimal8(uint16_t x, uint16_t y, uint8_t value);
    uint8_t DisplayDecimal16(uint16_t x, uint16_t y, uint16_t value);
    uint8_t DisplayDecimal32(uint16_t x, uint16_t y, uint32_t value);
    uint8_t DisplayDecimalSigned8(uint16_t x, uint16_t y, uint8_t value);
    uint8_t DisplayDecimalSigned16(uint16_t x, uint16_t y, uint16_t value);
    uint8_t DisplayDecimalSigned32(uint16_t x, uint16_t y, uint32_t value);
    uint8_t DisplayHex8(uint16_t x, uint16_t y, uint8_t value);
    uint8_t DisplayHex16(uint16_t x, uint16_t y, uint16_t value);
    uint8_t DisplayHex32(uint16_t x, uint16_t y, uint32_t value);
    uint8_t DisplayHex64(uint16_t x, uint16_t y, uint64_t value);
    uint8_t DisplayBinary8(uint16_t x, uint16_t y, uint8_t value);
    uint8_t DisplayBinary16(uint16_t x, uint16_t y, uint16_t value);
  
    uint8_t NumericalConfig(uint8_t config);
    parameter config (bit mask):
    NCONF_NoPlus      : display no plus (DisplayDecimalSigned)
    NCONF_NoZero      : display no leading zero (DisplayDecimal, DisplayDecimalSigned)
    NCONF_LeftAdjust  : display left adjust (DisplayDecimal, DisplayDecimalSigned)
    
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/
  
void Demo_DisplayNumbers(void)
{
  uint16_t u;
  uint8_t i, j;
  
  Display_Title(PSTR("Numerals"));
  delay(1500);

  for (i=0; i<4; i++)
  {
    	
	// change configuration 
        display.SetPenColor(255, 0, 0);
	if (i == 1) {
		display.DisplayText_F(5, 300, PSTR("No plus"));
		j = NCONF_NoPlus;
	} else if (i == 2) {
		display.DisplayText_F(5, 300, PSTR("No plus, no zero"));
		j = NCONF_NoPlus | NCONF_NoZero;
	} else if (i == 3) {
		display.DisplayText_F(5, 300, PSTR("No plus, no zero, left adj."));
		j = NCONF_NoPlus | NCONF_NoZero | NCONF_LeftAdjust;
	} else {
		j = 0;
	}
        display.SetPenColor(0, 0, 0);


	display.NumericalConfig(j);
	
	
	// display title	
        display.SetBrushColor(180, 180, 255);
	
	display.DisplayText_F(5, 35, PSTR("DECIMAL UNSIGNED"));
	display.DisplayText_F(5, 95, PSTR("DECIMAL SIGNED"));
	display.DisplayText_F(5, 155, PSTR("BINARY"));
	display.DisplayText_F(5, 200, PSTR("HEXADECIMAL"));
	
    display.SetBrushColor(255, 255, 255);
	
    for (u=50; u<300; u++)
	{

      display.DisplayDecimal8(5, 50, 240+(uint8_t)u);	
      display.DisplayDecimal16(5, 65, 2600+u);	
      display.DisplayDecimal32(5, 80, 56000+u);
	 
      display.DisplayDecimalSigned8(5, 110, 240+(uint8_t)u);		
      display.DisplayDecimalSigned16(5, 125, 2600+u);	
      display.DisplayDecimalSigned32(5, 140, 56000+u);
	  
      display.DisplayBinary8(5, 170, 240+(uint8_t)u);	
      display.DisplayBinary16(5, 185, 2600+u);

      display.DisplayHex8(5, 215, 240+(uint8_t)u);	
      display.DisplayHex16(5, 230, 2600+u);
      display.DisplayHex32(5, 245, 56000+u);
      display.DisplayHex64(5, 260, 135000+u);

    }

  }

  display.NumericalConfig(0);
  delay(2000);
  
}

  /********************************************************
  Section:
    User Flash Memory
  
  Related functions:
  
    uint8_t DisplayImagePackageImage(uint16_t x, uint16_t y, uint8_t packageindex, uint16_t itemindex);
    parameter: x, y, packageindex (0..63), itemindex (0..65535)
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/

void Demo_UserFlashMemory_Image(void)
{

  /* Display some images from user flash memory */

  Display_Title(PSTR("Images from user flash memory"));
  delay(3000);

  for (uint8_t i=0; i<10; i++)
  {
    display.DisplayImagePackageImage(0, 0, 63, i);  // Package 63
    delay(2000);
  } 
   
  delay(3000);
  display.FillScreen_BrushColor();

}

  /********************************************************
  Section:
    MicroSDCard
  
  Related functions:
  
    uint8_t DisplaySDCardImage(uint16_t x, uint16_t y, unsigned char *filename);
    parameter: x, y, *filename (points to string in sram)
    return: dsSuccess, dsTimeout, dsBusy (display status)    
    
    uint8_t DisplaySDCardImage_F(uint16_t x, uint16_t y, const char *filename);
    parameter: x, y, *filename (points to string in flash memory)    
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/

void Demo_MicroSDCard_Image(void)
{
  unsigned char s[10];
  
  // Display some images from micro sd card (FAT32)
  
  // Note:  (1) Maximum length of file name string is 12 characters plus final
  //            character NUL (0)
  //            Format is 8.3 (NAME.EXT, ex. "image1.bmp", "87654321.bmp")
  //        (2) Image format: Windows bitmap 24bit (.bmp)
  //        (3) If image overlaps display dimensions, command will be skipped

  Display_Title(PSTR("Images from micro sd card"));
  delay(3000);

  for (uint8_t i=1; i<8; i++)
  {
	  
    CreateFileName(s, i);      // creates filename "00iii.bmp", i = decimal index
	                       // Example: "00000.bmp", "00001.bmp", "00002.bmp" ...
    display.DisplaySDCardImage(0, 0, s); // if image overlaps display dimensions, command will be skipped
    
    display.DisplayText(5, DISPLAY_MODULE_HEIGHT-15, s);
    delay(2000);
	
  }  
  
  // file names stored in flash memory
  display.DisplaySDCardImage_F(0, 0, PSTR("00008.bmp"));
  display.DisplayText_F(5, DISPLAY_MODULE_HEIGHT-15, PSTR("00008.bmp")); delay(2000);
  
  display.DisplaySDCardImage_F(0, 0, PSTR("00009.bmp"));
  display.DisplayText_F(5, DISPLAY_MODULE_HEIGHT-15, PSTR("00009.bmp")); delay(2000);
    
  display.DisplaySDCardImage_F(0, 0, PSTR("00010.bmp")); 
  display.DisplayText_F(5, DISPLAY_MODULE_HEIGHT-15, PSTR("00010.bmp")); delay(2000);    
        
  delay(3000);
  display.FillScreen_BrushColor();

}

  /********************************************************
  Section:
    Draw Image Data
  
  Related functions:
    uint8_t DrawImage_Begin(uint16_t x, uint16_t y, uint16_t width, uint16_t height);
    void DrawImage_WritePixel(uint8_t r, uint8_t g, uint8_t b);
    void DrawImage_Stop(void);

    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/

void Demo_DrawImageData(void)
{

  Display_Title(PSTR("Draw Image Data"));
  delay(2000);

  // draw image data inside a rectangle
  
  // (1) define a rectangle: Display_DrawImage_Begin(uint16_t x, uint16_t y, uint16_t width, uint16_t height)
  // (2) send pixel data (RGB), image structure: row by row (address pointer of rows and columns will be auto incremented)
  // (3) stop command by dis asserting display module: Display_DrawImage_Stop()
  
  if ( dsSuccess == display.DrawImage_Begin(20, 30, 100, 120) )  // width = 100px, height = 120px
  {
	
	// create rgb data for demonstration
	for (uint8_t col=0; col<120; col++)
	{
		
	  for (uint8_t row=0; row<100; row++)  
            display.DrawImage_WritePixel(row<<1, col<<1, 255-col); // R G B
	}	  
	  
  }
  
  display.DrawImage_Stop();
  
  delay(3000);
  display.FillScreen_BrushColor();  

}


  /********************************************************
  Section:
    Touchpanel
  
  Related functions:
  
    uint8_t TouchPanel_SetConfig(uint8_t config)
    parameter: TOUCH_EVENT_CONT | TOUCH_EVENT_DOWN | TOUCH_EVENT_UP 
    return: dsSuccess, dsTimeout, dsBusy (display status)
    
    uint8_t TouchPanel_ReadEvent(void)
    return: dsSuccess, dsTimeout, dsBusy (display status)  
            Display.Events  
               
    uint8_t TouchPanel_ReadCont(void)
    return: dsSuccess, dsTimeout, dsBusy (display status) 
            Display.TouchX, Display.TouchY
              
    uint8_t TouchPanel_ReadDown(void)
    return: dsSuccess, dsTimeout, dsBusy (display status)
            Display.TouchDownX, Display.TouchDownY
            
    uint8_t TouchPanel_ReadUp(void)
    return: dsSuccess, dsTimeout, dsBusy (display status)
            Display.TouchUpX, Display.TouchUpY    
  ********************************************************/

void Demo_Touchpanel(void)
{
  uint8_t status;

#if DISPLAY_USE_TOUCHPANEL == true  
  // Display Buttons
   
  display.FillScreen_BrushColor(); 
  display.DrawRect_RGBColorFill(0, 280, 239, 319, 0xF0, 0xF0, 0xF0);   // filled rectangle (gray)   
  
  display.SetBrushColor(0xE8, 0xF1, 0xF5);   // Color of Button
    
  display.DisplayImagePackageImage(40, 290, 0, 3);  // Left Button, Button Image 66x21
  
  display.DisplayText_F(55, 296, PSTR("Clear"));   // Button text
    
  display.DisplayImagePackageImage(239-40-66, 290, 0, 3);     // Right Button, Button Image 66x21
  
  display.DisplayText_F(239-40-66 + 10, 296, PSTR("Cancel")); // Button text
   
  display.SetBrushColor(255, 255, 255);
  
  display.DisplayText_F(24, 100, PSTR("Touch to draw some lines"));  
    
  // Enable touch panel events continuous, down and up
  display.TouchPanel_SetConfig(TOUCH_EVENT_CONT | TOUCH_EVENT_DOWN | TOUCH_EVENT_UP);
  status = display.TouchPanel_ReadEvent();  // flush event register
  
  uint8_t proceed = 1;
  
  while (proceed)
  {  
  
    if (TOUCHPANEL_EVENT)
    {  

      if ( dsSuccess == display.TouchPanel_ReadEvent() )
      {
	  
	    if (Display.Events & TOUCH_EVENT_DOWN)
	    {
          if ( dsSuccess == display.TouchPanel_ReadDown() )  
		  {
		    if (280 > Display.TouchDownY)
		    {			  
		      display.DrawCircle_RGBColor(Display.TouchDownX, Display.TouchDownY, 10, 255, 0, 0);	
		      display.MoveTo(Display.TouchDownX, Display.TouchDownY);
		    } else {
			  // touch on buttons?
			  if ((Display.TouchDownX > 40) && (Display.TouchDownX < (40+66)) && (Display.TouchDownY > 290) && (Display.TouchDownY < (290+21)))
			  {
			    // Button clear touched, clear drawing area		 
			    display.DrawRect_BrushColorFill(0, 0, 239, 279);				
			  }			  
			  
			  if ((Display.TouchDownX > 133) && (Display.TouchDownX < (133+66)) && (Display.TouchDownY > 290) && (Display.TouchDownY < (290+21)))
			  {
			    // Button cancel touched, exit this demo section	
			    proceed = 0;
			  }			  			  
		    }		  	  	
		  }		
	    }
	  
	    if (Display.Events & TOUCH_EVENT_UP)
	    {
		  if ( dsSuccess ==  display.TouchPanel_ReadUp() )
		  {
		    if (280 > Display.TouchUpY)
		    {			
			  display.DrawCircle_RGBColor(Display.TouchUpX, Display.TouchUpY, 10, 0, 0, 255);
		    }		  
		  }	
	    }	  
	  
	    if (Display.Events & TOUCH_EVENT_CONT)
	    {
		  if ( dsSuccess ==  display.TouchPanel_ReadCont() )		
		  {
		    if (280 > Display.TouchY)
		    {			
			  //Display_DrawPixel_PenColor(Display.TouchX, Display.TouchY);
			  display.LineTo_PenColor(Display.TouchX, Display.TouchY);
		    }		  
		  }				   
	    }	  	  	 
      } // ReadEvent
    } // TOUCHPANEL_EVENT
  } // while
#endif
}


  /********************************************************
  Section:
    Embedded Font
  
  Related functions:

    uint8_t SetFont(uint8_t index);  // Set font for numbers and text
    uint8_t SetTextFont(uint8_t index); // Set font for text
    uint8_t SetNumberFont(uint8_t index); // Set font for numbers
  
    parameter:
    index: index of embedded font (0..MAX_EmbeddedFontCount)
    (MAX_EmbeddedFontCount = 17, INIT_EmbeddedFontIndex = 1)
    
    return: dsSuccess, dsTimeout, dsBusy (display status)
  ********************************************************/
void Demo_EmbeddedFont(void)
{
  uint16_t x, y;
  uint8_t j;
	
  Display_Title(PSTR("Embedded Font"));
  delay(1500);
  
  for (uint8_t i = 0; i < MAX_EmbeddedFontCount; i++)
  {
	  
	display.SetFont(i);
	
	x = 5;
        y = 30;
	for (j='!'; j<('z'+1); j++)
	{
		
	  if (x > DISPLAY_MODULE_WIDTH)
	  {
	    x = 5;
		y += 20;  
	  }	 	
		
      if (y < DISPLAY_MODULE_HEIGHT)
	  {  
		  
		display.DrawRect_BrushColorFill(x, y, x+20, y+20);  // clear display area 20x20px, previous char possibly higher and/or wider than next char
		  
		char s[2];
		s[0] = j;
		s[1] = 0;
		display.DisplayText(x, y, (unsigned char *)s);
	  }	  		
		
      x += 20;		
				
	}
	
	delay(1000);
	  
  }

  // setting font index for number font and text font individually
  display.SetNumberFont(INIT_EmbeddedFontIndex);  // restore font index
  display.SetTextFont(INIT_EmbeddedFontIndex);    // restore font index

  // or ...

  // setting font index for number font and text font together
  display.SetFont(INIT_EmbeddedFontIndex);        // restore font index

}


  /********************************************************
  Section:
    User Flash Memory Font
  
  Related functions:
    uint8_t SetUserFlashMemoryFont(uint8_t PackageIndex, uint8_t CharacterAlignMode);
    uint8_t SetUserFlashMemoryFont_Text(uint8_t PackageIndex, uint8_t CharacterAlignMode);
    uint8_t SetUserFlashMemoryFont_Number(uint8_t PackageIndex, uint8_t CharacterAlignMode);
  
    parameters:
    PackageIndex
    CharacterAlignMode: caProp, caLeft, caCenter, caRight
    caProp  : proportional character align
    caLeft  : left align, constant width
    caCenter: center align, constant width (best fit for numbers)
    caRight : right align, constant width
    
    return: dsSuccess, dsTimeout, dsBusy (display status)
  
  ********************************************************/

void Demo_UserFlashMemory_Font(void)
{
  uint16_t x, y;
  uint8_t j, m, i;
  
  Display_Title(PSTR("Font in user flash memory"));
  delay(2000);

  y = 40;
  display.DisplayText_F(5, y, PSTR("Embed your own fonts!"));  
  y += 20;
  display.DisplayText_F(5, y, PSTR("The characters will be displayed"));
  y += 20;
  display.DisplayText_F(5, y, PSTR("> antialiased"));
  delay(2000);
  
  y += 20;
  display.DisplayText_F(5, y, PSTR("> with proportional or constant"));  
  y += 10;
  display.DisplayText_F(5, y, PSTR("  char width"));
  delay(2000);
    
  y += 20;
  display.DisplayText_F(5, y, PSTR("> with chars adjusted to left,"));  
  y += 10;
  display.DisplayText_F(5, y, PSTR("  center or right")); 
  
  delay(5000);
  
  y += 50;
  display.DisplayText_F(5, y, PSTR("Displaying some font examples."));
  y += 20;
  display.DisplayText_F(5, y, PSTR("Switch to landscape mode ..."));
  delay(2000);  
  
  display.FillScreen_BrushColor();
  
  display.SetDisplayMode(dmLandscape); // switch to landscape mode to display big size number fonts
  display.DisplayText_F(5, 5,  PSTR("Number Font Examples"));  
  display.DisplayText_F(5, 25,  PSTR("Font sizes used in this example"));
  display.DisplayText_F(5, 35,  PSTR("approximately 80x100 pixels"));
    
  j = 0;  // display decimal8 content of this var
  m = 0;  // counter for font change
  i = 1;  // package index 1..7
  do
  {
	  if (0 == m)
	  {
		m = 50;
	    display.DrawRect_BrushColorFill(0, 70, 319, 219);
		if (i<8)
	      display.SetUserFlashMemoryFont_Number(i, caCenter); // Package Index, Character align: center (best for numerical)  
		i++;
	  }
      m--;
	  
	  display.DisplayDecimal8(30, 70, j);  
	  j++;	  
  } while (i < 8);


  display.FillScreen_BrushColor(); 
  display.DisplayText_F(5, 5,  PSTR("Text Font Examples"));
  display.DisplayText_F(5, 25,  PSTR("Font sizes used in this example"));
  display.DisplayText_F(5, 35,  PSTR("approximately 32x40 pixels"));
  
  for (i=9; i<49; i++)
  {
	  display.DrawRect_BrushColorFill(0, 70, 319, 219);
	  display.SetUserFlashMemoryFont_Text(i, caProp); // Package Index, Character align: proportional 
	  	 
	  display.DisplayText_F(5, 70, PSTR("This is a font example"));  
	  delay(500);
  
  }   
 
  delay(2000);
  
  display.DrawRect_BrushColorFill(0, 25, 319, 219);
  display.SetFont(INIT_EmbeddedFontIndex);   // restore font index, use embedded font  
 
  display.DisplayText_F(5, 70,  PSTR("Switch to portrait mode ..."));  
     
  delay(3000);
  display.SetDisplayMode(dmPortrait);   
  
  for (i=9; i<49; i++)
  {
	  display.FillScreen_BrushColor();
	  display.SetUserFlashMemoryFont_Text(i, caProp); // Package Index, Character align: proportional
	  
	 
	  display.DisplayText_F(1, 70, PSTR("This is a font example"));  
	  delay(500);
  
  }     
  
  delay(2000);  
  
  
  /* display ISO 8859 characters (0x20 .. 0xFF)*/
  
  display.FillScreen_BrushColor();
  display.SetFont(INIT_EmbeddedFontIndex);   // restore font index, use embedded font
  display.DisplayText_F(5, 5, PSTR("Full character set: ISO 8859"));  	    
  delay(3000);    

  display.SetUserFlashMemoryFont_Text(40, caProp); // Package Index, Character align: proportional
  display.DrawRect_BrushColorFill(0, 15, 239, 319);
  x = 5;
  y = 30;
  j = 0x20; // first visible character in ISO 8859 (0x20 .. 0xFF)
  do
  {
		    
		    if (x > (DISPLAY_MODULE_WIDTH-32))
		    {
			    x = 5;
			    y += 32;
		    }
		    
		    if (y < (DISPLAY_MODULE_HEIGHT-40))
		    {			   
			    
			    char s[2];
			    s[0] = j;
			    s[1] = 0;
			    display.DisplayText(x, y, (unsigned char *)s);
			    j++;	
		    } else {
				delay(5000);
				display.DrawRect_BrushColorFill(0, 15, 239, 319);
				x = 5;
				y = 30;
			}
		    
		    x += 32;

  } while (j != 0);
	    
  delay(5000);
	    
  display.SetFont(INIT_EmbeddedFontIndex);   // restore font index, use embedded font
  
}


  /********************************************************
  Section:
    Software Reset Display Module
  
  Note:
    After power on, display modul performs a software
    reset. So you do not have to software reset the
    display module afer system start.
    
    Software reset initializes configuration registers:
    > Pen color: clBlack
    > Brush color: clWhite
    > Numerical config: leading zero, left adjust, plus
    > Touchpanel events disabled
    > Embedded font index 1 for text and numbers
    > Backlight off
    
  Related functions:
    uint8_t SoftwareReset(void);  
    
    return: dsSuccess, dsTimeout, dsBusy (display status)  
  ********************************************************/

void Demo_SoftwareResetDisplayModule(void)
{
	
  // Software reset of display module
  // set system parameters to initialized state

  // recommended sequence to perform software reset:
  
  display.SoftwareReset();
  delay(100);
  while (DISPLAY_IS_BUSY) { nop(); }
  
  
  // Display module is ready to receive commands
  // Backlight on
  display.SetBacklightIntensity(255);	
  display.DisplayText_F(5, 5, PSTR("Softwar reset successful")); 
  display.DisplayText_F(5, 15, PSTR("executed."));   	    	
  delay(3000);	
  
}





/*
  Display_Title
  
  Displays title bar
*/

void Display_Title(const char *s)
{

	// clear screen
	display.FillScreen_BrushColor();
	
	// fill rectangle with color 0, 0, 255: light blue
        // pen color: white
        display.SetPenColor(255, 255, 255);
	display.SetBrushColor(100, 100, 255);
	display.DrawRect_BrushColorFill(2, 2, 237, 16);
	
	// display titel
	display.DisplayText_F(5, 5, s);

	// restore brush color and pen color
        display.SetPenColor(0, 0, 0);
	display.SetBrushColor(255, 255, 255);	
}

/*
  CreateFileName
  
  creates filename s="00nnn.bmp",  n = decimal index
 
  Example: "00000.bmp", "00001.bmp", "00002.bmp" ...
*/
void CreateFileName(unsigned char *s, uint8_t index)
{
	
	uint8_t d;

	s[0] = '0';
	s[1] = '0';
	s[2] = '0';  // 100
	s[3] = '0';  //  10
	s[4] = '0';  //   1
	s[5] = '.';
	s[6] = 'b';
	s[7] = 'm';
	s[8] = 'p';
	s[9] = 0;

	for( d = '0'; index >= 100; index -= 100) d++;  // 100
	s[2] = d;

	for( d = '0'; index >= 10; index -= 10) d++;    // 10
	s[3] = d;

	s[4] = (uint8_t)index+'0';                      // 1

}
