/**********************************************************************************
* This is an Arduino library for the display module families
* TFT2428T, TFT50D and TFT60D from XV-Display.de
*
* Revision 1
*
* Recommended development environment:
* Arduino environment
* ---> http://arduino.cc/en/Main/Software
* E.g. one of following Boards:
* Arduino Mega 2560 R3
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/Arduino-Mega-2560.html
* Arduino Uno Rev3
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/Arduino-Uno-Rev3.html
* ITEADUINO MEGA 2560
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/ITEADUINO-MEGA-2560.html
* 
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
* 
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
* 
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* Arduino boards
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
* 
* 
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file. 
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/

#include <avr/pgmspace.h>

#include "XVDisplay_Library.h"
#include "Config.h"

XV_Display_t Display;

void _WriteDummyByte (void);
void _WriteData8 (void);
void _WriteData16 (void);
void _WriteData32 (void);
void _WriteData64 (void);
void _WriteRGB (void);
void _WriteXY1 (void);
void _WriteXY1RGB (void);
void _WriteXY12 (void);
void _WriteXY12RGB (void);
void _WriteXY1R (void);
void _WriteXY1RRGB (void);
void _WriteXY1Data8 (void);
void _WriteXY1Data16 (void);
void _WriteXY1Data32 (void);
void _WriteXY1Data64 (void);
void _WriteXY1PaItem (void);
void _WriteXY1Sting (void);
void _WriteXY1StingF (void);
void _WritePaItem(void);
void _WritePaCa(void);
void _WriteX1 (void);
void _WriteXY1WH (void);
#if DISPLAY_USE_TOUCHPANEL == true
void _WriteEvent (void);
void _ReadEvent (void);
void _ReadXYUp (void);
void _ReadXYDown (void);
void _ReadXY (void);
#endif
	
void _SPI_Write16( uint16_t data16 );
void _SPI_Write32( uint32_t data32 );
void _SPI_Write64( uint64_t data64 );
uint16_t _SPI_Read16( void );
uint8_t _SPI_RW( uint8_t output );		

uint8_t _WaitWhileDisplayBusy_Timeout500ms( void );
uint8_t _WaitWhileDisplayBusy( void );
uint8_t _WaitUntilDisplayBusy( void );
#if DISPLAY_USE_TOUCHPANEL == true
uint8_t _WaitWhileTouch( void );
#endif
	
xvdisplay::xvdisplay(  )
{
	
  Init_Hardware();

  /***
  variable init
  ***/  	
  Display.X1 = 0;
  Display.Y1 = 0;
  Display.X2 = 0;
  Display.Y2 = 0;
  Display.Data8 = 0;
  Display.Data16 = 0;
  Display.Data32 = 0;
  Display.Data64 = 0;
  Display.Rad = 0;
  Display.Width = 0;
  Display.Height = 0;
  Display.ColorR = 0;
  Display.ColorG = 0;
  Display.ColorB = 0;
  Display.PackageIndex = 0;
  Display.ItemIndex = 0;
  #if DISPLAY_USE_TOUCHPANEL == true
  Display.Events = 0;
  Display.TouchX = 0;
  Display.TouchY = 0;
  Display.TouchUpX = 0;
  Display.TouchUpY = 0;
  Display.TouchDownX = 0;
  Display.TouchDownY = 0;
  #endif
}


void xvdisplay::Init_Hardware(void)
{

  /***
  DISPLAY CONTROL SIGNALS
  ***/
  
  pinMode(DISPLAY_SIGNAL_BUSY, INPUT_PULLUP); // BUSY: Input, pullup activated
  
  digitalWrite(DISPLAY_SIGNAL_CS, LOW);       // CS: Output, high
  pinMode(DISPLAY_SIGNAL_CS, OUTPUT);

#if DISPLAY_USE_TOUCHPANEL == true
  pinMode(DISPLAY_SIGNAL_TINT, INPUT_PULLUP); // TINT: Input, pullup activated    
#endif


  /***
  Software SPI
  ***/
  
  // MOSI, SCK: Output, high

  digitalWrite(DISPLAY_SPI_MOSI, HIGH);       // MOSI: Output, high
  pinMode(DISPLAY_SPI_MOSI, OUTPUT);

  digitalWrite(DISPLAY_SPI_SCK, HIGH);        // SCK: Output, high
  pinMode(DISPLAY_SPI_SCK, OUTPUT);

  pinMode(DISPLAY_SPI_MISO, INPUT);           // MISO: Input, high impedance state 

}  


/****
 Graphical Commands
****/

uint8_t xvdisplay::FillScreen_BrushColor ( void )
{
	
  return Command (CMD_FillScreen_BrushColor, _WriteDummyByte);
	
}

uint8_t xvdisplay::FillScreen_PenColor ( void )
{
	
	return Command (CMD_FillScreen_PenColor, _WriteDummyByte);
	
}

uint8_t xvdisplay::FillScreen_RGBColor ( uint8_t r, uint8_t g, uint8_t b )
{

  Display.ColorR = r;
  Display.ColorG = g;
  Display.ColorB = b;

  return Command (CMD_FillScreen_RGBColor, _WriteRGB);
 
}

uint8_t xvdisplay::MoveTo(uint16_t x, uint16_t y)
{
	
  Display.X1 = x;
  Display.Y1 = y;
  
  return Command (CMD_MoveTo, _WriteXY1);	
}

uint8_t xvdisplay::LineTo_PenColor(uint16_t x, uint16_t y)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	
	return Command (CMD_LineTo_PenColor, _WriteXY1);
}

uint8_t xvdisplay::LineTo_BrushColor(uint16_t x, uint16_t y)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	
	return Command (CMD_LineTo_BrushColor, _WriteXY1);
}

uint8_t xvdisplay::LineTo_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b)
{
	
  Display.X1 = x;
  Display.Y1 = y;
  Display.ColorR = r;
  Display.ColorG = g;
  Display.ColorB = b;	

  return Command (CMD_LineTo_RGBColor, _WriteXY1RGB);
}

uint8_t xvdisplay::DrawPixel_PenColor(uint16_t x, uint16_t y)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	
	return Command (CMD_DrawPixel_PenColor, _WriteXY1);
}

uint8_t xvdisplay::DrawPixel_BrushColor(uint16_t x, uint16_t y)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	
	return Command (CMD_DrawPixel_BrushColor, _WriteXY1);
}

uint8_t xvdisplay::DrawPixel_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.ColorR = r;
	Display.ColorG = g;
	Display.ColorB = b;

	return Command (CMD_DrawPixel_RGBColor, _WriteXY1RGB);
}

uint8_t xvdisplay::DrawLine_PenColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	
    Display.X1 = x1;
    Display.Y1 = y1;
    Display.X2 = x2;
    Display.Y2 = y2;	
	
    return Command (CMD_DrawLine_PenColor, _WriteXY12);
}

uint8_t xvdisplay::DrawLine_BrushColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	
	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	
	return Command (CMD_DrawLine_BrushColor, _WriteXY12);
}

uint8_t xvdisplay::DrawLine_RGBColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b)
{
	
	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	Display.ColorR = r;
	Display.ColorG = g;
	Display.ColorB = b;
	
	return Command (CMD_DrawLine_RGBColor, _WriteXY12RGB);
}

uint8_t xvdisplay::DrawRect_PenColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	
	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	
	return Command (CMD_DrawRect_PenColor, _WriteXY12);
}

uint8_t xvdisplay::DrawRect_PenColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	
	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	
	return Command (CMD_DrawRect_PenColorFill, _WriteXY12);
}

uint8_t xvdisplay::DrawRect_BrushColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	
	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	
	return Command (CMD_DrawRect_BrushColor, _WriteXY12);
}

uint8_t xvdisplay::DrawRect_BrushColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
	
	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	
	return Command (CMD_DrawRect_BrushColorFill, _WriteXY12);
}

uint8_t xvdisplay::DrawRect_RGBColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b)
{
	
	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	Display.ColorR = r;
	Display.ColorG = g;
	Display.ColorB = b;	
	
	return Command (CMD_DrawRect_RGBColor, _WriteXY12RGB);
}

uint8_t xvdisplay::DrawRect_RGBColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b)
{

	Display.X1 = x1;
	Display.Y1 = y1;
	Display.X2 = x2;
	Display.Y2 = y2;
	Display.ColorR = r;
	Display.ColorG = g;
	Display.ColorB = b;
	
	return Command (CMD_DrawRect_RGBColorFill, _WriteXY12RGB);
	
}

uint8_t xvdisplay::DrawCircle_PenColor(uint16_t x, uint16_t y, uint8_t rad)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Rad = rad;
	
	return Command (CMD_DrawCircle_PenColor, _WriteXY1R);
}

uint8_t xvdisplay::DrawCircle_BrushColor(uint16_t x, uint16_t y, uint8_t rad)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Rad = rad;
	
	return Command (CMD_DrawCircle_BrushColor, _WriteXY1R);
}

uint8_t xvdisplay::DrawCircle_RGBColor(uint16_t x, uint16_t y, uint8_t rad, uint8_t r, uint8_t g, uint8_t b)
{

	Display.X1 = x;
	Display.Y1 = y;
	Display.Rad = rad;
	Display.ColorR = r;
	Display.ColorG = g;
	Display.ColorB = b;
	

	return Command (CMD_DrawCircle_RGBColor, _WriteXY1RRGB);

}

uint8_t xvdisplay::DrawImage_Begin(uint16_t x, uint16_t y, uint16_t width, uint16_t height)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Width = width;
	Display.Height = height;
	
		
  uint8_t status = _WaitWhileDisplayBusy();

  if ( dsSuccess == status )
  {

    digitalWrite(DISPLAY_SIGNAL_CS, LOW);// assert display module
    DELAY_BEFORECMD

    _SPI_Write16(CMD_DrawImage_Begin);  // write command word
    DELAY_AFTERCMD
    status = _WaitUntilDisplayBusy();   // wait until display module responds busy low
    if ( dsSuccess == status )
	{
      // execute display protocol process
      _WriteXY1WH();        	 
	}
  }
  
  if (dsSuccess != status) digitalWrite(DISPLAY_SIGNAL_CS, HIGH); // disassert display module
  
  _WaitWhileDisplayBusy();             // wait until display module responds busy high

  return status;			
		
}

void xvdisplay::DrawImage_WritePixel(uint8_t r, uint8_t g, uint8_t b)
{

	_SPI_RW(r);
	_SPI_RW(g);	
	_SPI_RW(b);	

}

void xvdisplay::DrawImage_Stop(void)
{
   digitalWrite(DISPLAY_SIGNAL_CS, HIGH);	// disassert xvdisplay module
   delayMicroseconds(1);
}


/****
 System Commands
****/

uint8_t xvdisplay::SoftwareReset(void)
{
	
	Display.Data16 = CMD_SoftwareReset_A;
	
	return Command (CMD_SoftwareReset, _WriteData16);
	
}

uint8_t xvdisplay::SetBacklightIntensity(uint8_t intensity)
{
	
	Display.Data8 = intensity;
	
	return Command (CMD_SetBacklightIntensity, _WriteData8);
}

uint8_t xvdisplay::SetDisplayMode(uint8_t mode)
{
	
	Display.Data8 = mode;
	
	return Command (CMD_SetDisplayMode, _WriteData8);
	
}

uint8_t xvdisplay::SetPenColor(uint8_t r, uint8_t g, uint8_t b)
{
	
	Display.ColorR = r;
	Display.ColorG = g;
	Display.ColorB = b;
	
	return Command (CMD_SetPenColor, _WriteRGB);
	
}

uint8_t xvdisplay::SetBrushColor(uint8_t r, uint8_t g, uint8_t b)
{
	
	Display.ColorR = r;
	Display.ColorG = g;
	Display.ColorB = b;
	
	return Command (CMD_SetBrushColor, _WriteRGB);
	
}


uint8_t xvdisplay::SetPenColor_Index(uint32_t color)
{
	
	return SetPenColor( (uint8_t)(color), (uint8_t)(color>>8), (uint8_t)(color>>16) ); // r g b
	
}

uint8_t xvdisplay::SetBrushColor_Index(uint32_t color)
{
	
	return SetBrushColor( (uint8_t)(color), (uint8_t)(color>>8), (uint8_t)(color>>16) ); // r g b
	
}



/****
 Numerical Commands
****/

uint8_t xvdisplay::DisplayDecimal8(uint16_t x, uint16_t y, uint8_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data8 = value;
	
	return Command (CMD_DisplayDecimal8, _WriteXY1Data8);	
	
}

uint8_t xvdisplay::DisplayDecimal16(uint16_t x, uint16_t y, uint16_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data16 = value;
	
	return Command (CMD_DisplayDecimal16, _WriteXY1Data16);
	
}

uint8_t xvdisplay::DisplayDecimal32(uint16_t x, uint16_t y, uint32_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data32 = value;
	
	return Command (CMD_DisplayDecimal32, _WriteXY1Data32);
	
}

uint8_t xvdisplay::DisplayDecimalSigned8(uint16_t x, uint16_t y, uint8_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data8 = value;
	
	return Command (CMD_DisplayDecimalSigned8, _WriteXY1Data8);	
	
}

uint8_t xvdisplay::DisplayDecimalSigned16(uint16_t x, uint16_t y, uint16_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data16 = value;
	
	return Command (CMD_DisplayDecimalSigned16, _WriteXY1Data16);
	
}

uint8_t xvdisplay::DisplayDecimalSigned32(uint16_t x, uint16_t y, uint32_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data32 = value;
	
	return Command (CMD_DisplayDecimalSigned32, _WriteXY1Data32);
	
}

uint8_t xvdisplay::DisplayHex8(uint16_t x, uint16_t y, uint8_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data8 = value;
	
	return Command (CMD_DisplayHex8, _WriteXY1Data8);
	
}

uint8_t xvdisplay::DisplayHex16(uint16_t x, uint16_t y, uint16_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data16 = value;
	
	return Command (CMD_DisplayHex16, _WriteXY1Data16);
	
}

uint8_t xvdisplay::DisplayHex32(uint16_t x, uint16_t y, uint32_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data32 = value;
	
	return Command (CMD_DisplayHex32, _WriteXY1Data32);
	
}

uint8_t xvdisplay::DisplayHex64(uint16_t x, uint16_t y, uint64_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data64 = value;
	
	return Command (CMD_DisplayHex64, _WriteXY1Data64);
	
}

uint8_t xvdisplay::DisplayBinary8(uint16_t x, uint16_t y, uint8_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data8 = value;
	
	return Command (CMD_DisplayBinary8, _WriteXY1Data8);
	
}

uint8_t xvdisplay::DisplayBinary16(uint16_t x, uint16_t y, uint16_t value)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.Data16 = value;
	
	return Command (CMD_DisplayBinary16, _WriteXY1Data16);
	
}

uint8_t xvdisplay::NumericalConfig(uint8_t config)
{
	
	Display.Data8 = config;
		
	return Command (CMD_NumericalConfig, _WriteData8);
	
}


/****
 Text Commands
****/


uint8_t xvdisplay::DisplayText(uint16_t x, uint16_t y, unsigned char *s)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.String = (unsigned char *)s;
	
	return Command (CMD_DisplayText, _WriteXY1Sting);	
	
}

uint8_t xvdisplay::DisplayText_F(uint16_t x, uint16_t y, const char *s)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.String = (unsigned char *)s;
	
	return Command (CMD_DisplayText, _WriteXY1StingF);
	
}

uint8_t xvdisplay::SetTextMarginX(uint16_t x)
{
	
	Display.X1 = x;
	
	return Command (CMD_SetTextMarginX, _WriteX1);	
}


uint8_t xvdisplay::SetFont(uint8_t index)
{
	
	Display.Data8 = index;
	
	return Command (CMD_SetFont, _WriteData8);	
	
}

uint8_t xvdisplay::SetNumberFont(uint8_t index)
{
	
	Display.Data8 = index;
	
	return Command (CMD_SetNumberFont, _WriteData8);
	
}

uint8_t xvdisplay::SetTextFont(uint8_t index)
{
	
	Display.Data8 = index;
	
	return Command (CMD_SetTextFont, _WriteData8);
	
}

uint8_t xvdisplay::SetUserFlashMemoryFont(uint8_t PackageIndex, uint8_t CharacterAlignMode)
{
	
	Display.PackageIndex = PackageIndex;
	Display.Data8 = CharacterAlignMode;
	
	return Command (CMD_SetUserFlashMemoryFont, _WritePaCa);
	
}

uint8_t xvdisplay::SetUserFlashMemoryFont_Text(uint8_t PackageIndex, uint8_t CharacterAlignMode)
{
	
	Display.PackageIndex = PackageIndex;
	Display.Data8 = CharacterAlignMode;
	
	return Command (CMD_SetUserFlashMemoryFont_Text, _WritePaCa);
	
}

uint8_t xvdisplay::SetUserFlashMemoryFont_Number(uint8_t PackageIndex, uint8_t CharacterAlignMode)
{
	
	Display.PackageIndex = PackageIndex;
	Display.Data8 = CharacterAlignMode;
	
	return Command (CMD_SetUserFlashMemoryFont_Number, _WritePaCa);
	
}



/****
 User Flash Memory
****/


uint8_t xvdisplay::DisplayImagePackageImage(uint16_t x, uint16_t y, uint8_t packageindex, uint16_t itemindex)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.PackageIndex = packageindex;
	Display.ItemIndex = itemindex;
	
	return Command (CMD_DisplayImagePackage_Image, _WriteXY1PaItem);	
	
}

uint8_t xvdisplay::DisplaySDCardImage(uint16_t x, uint16_t y, unsigned char *filename)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.String = (unsigned char *)filename;
	
	return Command (CMD_DisplaySDCard_Image, _WriteXY1Sting);	
	
}

uint8_t xvdisplay::DisplaySDCardImage_F(uint16_t x, uint16_t y, const char *filename)
{
	
	Display.X1 = x;
	Display.Y1 = y;
	Display.String = (unsigned char *)filename;
	
	return Command (CMD_DisplaySDCard_Image, _WriteXY1StingF);
	
}


/****
 Touch Panel
****/

#if DISPLAY_USE_TOUCHPANEL == true

uint8_t xvdisplay::TouchPanel_SetConfig(uint8_t config)
{
	
	Display.Events = config;
		
	return Command (CMD_TouchPanel_SetConfig, _WriteEvent);	
	
}

uint8_t xvdisplay::TouchPanel_ReadEvent(void)
{
	
	return Command (CMD_TouchPanel_ReadEvent, _ReadEvent);
	
}


uint8_t xvdisplay::TouchPanel_Calibrate(void)
{
	
	return Command (CMD_TouchPanel_Calibrate, _WriteDummyByte);
	
}


uint8_t xvdisplay::TouchPanel_ReadCont(void)
{
	
	return Command (CMD_TouchPanel_ReadCont, _ReadXY);	
	
}

uint8_t xvdisplay::TouchPanel_ReadDown(void)
{
	
	return Command (CMD_TouchPanel_ReadDown, _ReadXYDown);
	
}

uint8_t xvdisplay::TouchPanel_ReadUp(void)
{
	
	return Command (CMD_TouchPanel_ReadUp, _ReadXYUp);
	
}

#endif







uint8_t xvdisplay::Command(uint16_t command, display_process process)
{	
  uint8_t status = _WaitWhileDisplayBusy();

  if ( dsSuccess == status )
  {

    digitalWrite(DISPLAY_SIGNAL_CS, LOW); // assert display module
    DELAY_BEFORECMD

    _SPI_Write16(command);				// write command word
    DELAY_AFTERCMD
    status = _WaitUntilDisplayBusy();	// wait until display module responds busy low
    if ( dsSuccess == status )
	{
      // execute display protocol process
      process();
      DELAY_BEFOREDEASSERT
      // Note: Deassert SS_ (CS_) not until the SPI-Transfer is really finished
      //       otherwise display module possibly does not receive the last data byte
      //       completely. We use an small delay of 1us to acheive this.   
      	  
	}
  }
   
  digitalWrite(DISPLAY_SIGNAL_CS, HIGH);       // disassert display module

  return status;	
		
}

void _WriteDummyByte (void)
{
   _SPI_RW(0xFF);     // Dummy Byte
}

void _WriteData8 (void)
{
  _SPI_RW(Display.Data8);
}

void _WriteData16 (void)
{
  _SPI_Write16(Display.Data16);
}

void _WriteData32 (void)
{
  _SPI_Write16(Display.Data32);
}

void _WriteData64 (void)
{
  _SPI_Write16(Display.Data64);
}


void _WriteRGB (void)
{
  _SPI_RW(Display.ColorR);
  _SPI_RW(Display.ColorG);
  _SPI_RW(Display.ColorB);
}


void _WriteXY1 (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
}

void _WriteXY1RGB (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_RW(Display.ColorR);
  _SPI_RW(Display.ColorG);
  _SPI_RW(Display.ColorB);
}


void _WriteXY12 (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_Write16(Display.X2);
  _SPI_Write16(Display.Y2);
}

void _WriteXY12RGB (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_Write16(Display.X2);
  _SPI_Write16(Display.Y2);
  _SPI_RW(Display.ColorR);
  _SPI_RW(Display.ColorG);
  _SPI_RW(Display.ColorB);
}

void _WriteXY1R (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_RW(Display.Rad);
}

void _WriteXY1RRGB (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_RW(Display.Rad);
  _SPI_RW(Display.ColorR);
  _SPI_RW(Display.ColorG);
  _SPI_RW(Display.ColorB);
}

void _WriteXY1Data8 (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_RW(Display.Data8);
}

void _WriteXY1Data16 (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_Write16(Display.Data16);
}

void _WriteXY1Data32 (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_Write32(Display.Data32);
}

void _WriteXY1Data64 (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_Write64(Display.Data64);
}

void _WriteXY1PaItem (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_RW(Display.PackageIndex);
  _SPI_Write16(Display.ItemIndex);
}

void _WriteXY1Sting (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
    
  uint8_t i = 0;
  while (Display.String[i] && (i<100))      // max. string length = 100
  {
    _SPI_RW(Display.String[i++]); 
  }
  _SPI_RW(0); 
}

void _WriteXY1StingF (void)
{
   _SPI_Write16(Display.X1);
   _SPI_Write16(Display.Y1);
    
    uint8_t c = pgm_read_byte(&(*Display.String++));
    uint8_t i = 0;
    while (c && (i<100))      // max. string length = 100
    {
     _SPI_RW(c); 
      i++; 
      c = pgm_read_byte(&(*Display.String++));
    }
   _SPI_RW(0); 
}

void _WritePaItem(void)
{
  _SPI_RW(Display.PackageIndex);
  _SPI_Write16(Display.ItemIndex);
}

void _WritePaCa(void)
{
  _SPI_RW(Display.PackageIndex);
  _SPI_RW(Display.Data8);         // Character align mode 
}

void _WriteX1 (void)
{
  _SPI_Write16(Display.X1);
}


void _WriteXY1WH (void)
{
  _SPI_Write16(Display.X1);
  _SPI_Write16(Display.Y1);
  _SPI_Write16(Display.Width);
  _SPI_Write16(Display.Height);
}

#if DISPLAY_USE_TOUCHPANEL == true
void _WriteEvent (void)
{
  _SPI_RW(Display.Events);
  Display.Events = 0;
}

void _ReadEvent (void)
{
  Display.Events = _SPI_RW(0xFF);
  _WaitWhileTouch();
}

void _ReadXYUp (void)
{
  Display.TouchUpX = _SPI_Read16();
  Display.TouchUpY = _SPI_Read16();
}

void _ReadXYDown (void)
{
	Display.TouchDownX = _SPI_Read16();
	Display.TouchDownY = _SPI_Read16();
}

void _ReadXY (void)
{
	Display.TouchX = _SPI_Read16();
	Display.TouchY = _SPI_Read16();
}
#endif


uint8_t _WaitWhileDisplayBusy(void)
{
  uint32_t time;

  time = 1000000;			// timeout 5s

  while ( digitalRead(DISPLAY_SIGNAL_BUSY) == LOW )
  {
    time--;
	if (0 == time) return dsTimeout;
    delayMicroseconds(5);
  }

  return dsSuccess;
}


uint8_t _WaitUntilDisplayBusy(void)
{
  uint16_t time;

  time = 50000;					// timeout 250ms

  while ( digitalRead(DISPLAY_SIGNAL_BUSY) == HIGH )
  {
    time--;
	if (0 == time) return dsTimeout;
    delayMicroseconds(5);
  }

  return dsSuccess;
}

#if DISPLAY_USE_TOUCHPANEL == true
uint8_t _WaitWhileTouch(void)
{
  uint16_t time;

  time = 10000;					// timeout 10ms

  while ( digitalRead(DISPLAY_SIGNAL_TINT) == LOW )
  {
    time--;
	if (0 == time) return dsTimeout;
    delayMicroseconds(1);
  }

  return dsSuccess;
}
#endif




void _SPI_Write16(uint16_t data16)
{
  _SPI_RW((uint8_t)(data16>>8));
  _SPI_RW((uint8_t) data16);
}

void _SPI_Write32(uint32_t data32)
{
  _SPI_RW((uint8_t)(data32>>24));
  _SPI_RW((uint8_t)(data32>>16));
  _SPI_RW((uint8_t)(data32>>8));
  _SPI_RW((uint8_t) data32);
}

void _SPI_Write64(uint64_t data64)
{
  _SPI_RW((uint8_t)(data64>>56));
  _SPI_RW((uint8_t)(data64>>48));
  _SPI_RW((uint8_t)(data64>>40));
  _SPI_RW((uint8_t)(data64>>32));
  _SPI_RW((uint8_t)(data64>>24));
  _SPI_RW((uint8_t)(data64>>16));
  _SPI_RW((uint8_t)(data64>>8));
  _SPI_RW((uint8_t) data64);
}

uint16_t _SPI_Read16(void)
{
  uint16_t result;
  result  = (uint16_t) _SPI_RW(0xFF)<<8;
  result |= _SPI_RW(0xFF);
  return result;
}


  /***
  SoftwareSPI
  
  Note: Maximum SPI frequency is 2MHz. Typical frequency achived by function below
        is 1,5MHz with a Xmega AVR at 32MHz systemclock. If using other systems, possibly
		you have to insert delays, to slow down SPI frequency.
  ***/

uint8_t _SPI_RW (uint8_t output)
{
    uint8_t input=0;

    uint8_t i;
	
    for (i=0; i<8; i++)
    {

      digitalWrite(DISPLAY_SPI_SCK, LOW);			// SCK low

      if (output & 0x80)
        digitalWrite(DISPLAY_SPI_MOSI, HIGH);		// MOSI high
      else
        digitalWrite(DISPLAY_SPI_MOSI, LOW);		// MOSI low

      digitalWrite(DISPLAY_SPI_SCK, HIGH);			// SCK high
      input <<= 1;									// Input MISO

      if (digitalRead(DISPLAY_SPI_MISO) == HIGH)
  	    input |= 1;
      else
	    input &= ~1;

      output <<= 1;
    }
	
    return input; //return the byte clocked SPI slave

}		

















