/**********************************************************************************
* This is an Arduino library for the display module families
* TFT2428T, TFT50D and TFT60D from XV-Display.de
*
* Revision 1
*
* Recommended development environment:
* Arduino environment
* ---> http://arduino.cc/en/Main/Software
* E.g. one of following Boards:
* Arduino Mega 2560 R3
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/Arduino-Mega-2560.html
* Arduino Uno Rev3
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/Arduino-Uno-Rev3.html
* ITEADUINO MEGA 2560
* ---> http://mikrocontroller-praxis.de/de/Development-Tools/Arduino/ITEADUINO-MEGA-2560.html
* 
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
* 
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
* 
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* Arduino boards
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
* 
* 
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file. 
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/

#ifndef XVDisplay_h
#define XVDisplay_h

#include "Arduino.h"
#include "Config.h"


#define DISPLAY_ASSERTED     digitalRead(DISPLAY_SIGNAL_CS) == HIGH
#define DISPLAY_NOT_ASSERTED digitalRead(DISPLAY_SIGNAL_CS) == LOW
#define DISPLAY_IS_BUSY      digitalRead(DISPLAY_SIGNAL_BUSY) == LOW
#define DISPLAY_IN_NOT_BUSY  digitalRead(DISPLAY_SIGNAL_BUSY) == HIGH

#if DISPLAY_USE_TOUCHPANEL == true
#define TOUCHPANEL_EVENT     digitalRead(DISPLAY_SIGNAL_TINT) == LOW
#endif
	
typedef void (*display_process) ( void );
#define ATTR_PACKED   __attribute__ ((packed))

typedef struct
{

  uint16_t  X1;        
  uint16_t  Y1;  

  uint16_t  X2;
  uint16_t  Y2;

  uint8_t  Data8;
  uint16_t Data16;
  uint32_t Data32;
  uint64_t Data64;

  uint8_t Rad;
  uint16_t Width;
  uint16_t Height;

  uint8_t  ColorR;
  uint8_t  ColorG;
  uint8_t  ColorB;
			
  uint8_t  PackageIndex;
  uint16_t ItemIndex;			
			
  #if DISPLAY_USE_TOUCHPANEL == true			
  uint8_t Events;
  uint16_t TouchUpX;
  uint16_t TouchUpY;
  uint16_t TouchDownX;
  uint16_t TouchDownY;
  uint16_t TouchX;
  uint16_t TouchY;
  #endif
 
  const unsigned char  *String;

} ATTR_PACKED XV_Display_t;

extern XV_Display_t Display;


enum Display_Status_t
{
  dsSuccess        = 0,
  dsTimeout        = 1,
  dsBusy           = 2,
};    


class xvdisplay
{
  public:
  xvdisplay();	  
  uint8_t SoftwareReset(void);
  uint8_t SetBacklightIntensity(uint8_t intensity);
  uint8_t SetDisplayMode(uint8_t mode);
  uint8_t DebugMode(uint8_t mode);
  uint8_t SetPenColor(uint8_t r, uint8_t g, uint8_t b);
  uint8_t SetBrushColor(uint8_t r, uint8_t g, uint8_t b);
  uint8_t SetPenColor_Index(uint32_t color);  // 00 bb gg rr
  uint8_t SetBrushColor_Index(uint32_t color); // 00 bb gg rr		
  uint8_t DisplayDecimal8(uint16_t x, uint16_t y, uint8_t value);
  uint8_t DisplayDecimal16(uint16_t x, uint16_t y, uint16_t value);
  uint8_t DisplayDecimal32(uint16_t x, uint16_t y, uint32_t value);
  uint8_t DisplayDecimalSigned8(uint16_t x, uint16_t y, uint8_t value);
  uint8_t DisplayDecimalSigned16(uint16_t x, uint16_t y, uint16_t value);
  uint8_t DisplayDecimalSigned32(uint16_t x, uint16_t y, uint32_t value);
  uint8_t DisplayHex8(uint16_t x, uint16_t y, uint8_t value);
  uint8_t DisplayHex16(uint16_t x, uint16_t y, uint16_t value);
  uint8_t DisplayHex32(uint16_t x, uint16_t y, uint32_t value);
  uint8_t DisplayHex64(uint16_t x, uint16_t y, uint64_t value);
  uint8_t DisplayBinary8(uint16_t x, uint16_t y, uint8_t value);
  uint8_t DisplayBinary16(uint16_t x, uint16_t y, uint16_t value);
  uint8_t NumericalConfig(uint8_t config);
  uint8_t FillScreen_BrushColor(void);
  uint8_t FillScreen_PenColor(void);
  uint8_t FillScreen_RGBColor(uint8_t r, uint8_t g, uint8_t b);
  uint8_t MoveTo(uint16_t x, uint16_t y);
  uint8_t LineTo_PenColor(uint16_t x, uint16_t y);
  uint8_t LineTo_BrushColor(uint16_t x, uint16_t y);
  uint8_t LineTo_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b);
  uint8_t DrawPixel_PenColor(uint16_t x, uint16_t y);
  uint8_t DrawPixel_BrushColor(uint16_t x, uint16_t y);
  uint8_t DrawPixel_RGBColor(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b);
  uint8_t DrawLine_PenColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
  uint8_t DrawLine_BrushColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
  uint8_t DrawLine_RGBColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b);
  uint8_t DrawRect_PenColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
  uint8_t DrawRect_PenColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
  uint8_t DrawRect_BrushColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
  uint8_t DrawRect_BrushColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
  uint8_t DrawRect_RGBColor(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b);
  uint8_t DrawRect_RGBColorFill(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint8_t r, uint8_t g, uint8_t b);
  uint8_t DrawCircle_PenColor(uint16_t x, uint16_t y, uint8_t radius);
  uint8_t DrawCircle_BrushColor(uint16_t x, uint16_t y, uint8_t radius);
  uint8_t DrawCircle_RGBColor(uint16_t x, uint16_t y, uint8_t radius, uint8_t r, uint8_t g, uint8_t b);
  uint8_t DrawImage_Begin(uint16_t x, uint16_t y, uint16_t width, uint16_t height);
  void DrawImage_WritePixel(uint8_t r, uint8_t g, uint8_t b);
  void DrawImage_Stop(void);
  uint8_t DisplayText(uint16_t x, uint16_t y, unsigned char *s);
  uint8_t DisplayText_F(uint16_t x, uint16_t y, const char *s);
  uint8_t SetTextMarginX(uint16_t x);
  uint8_t SetFont(uint8_t index);
  uint8_t SetTextFont(uint8_t index);
  uint8_t SetNumberFont(uint8_t index);
  uint8_t SetUserFlashMemoryFont(uint8_t PackageIndex, uint8_t CharacterAlignMode);
  uint8_t SetUserFlashMemoryFont_Text(uint8_t PackageIndex, uint8_t CharacterAlignMode);
  uint8_t SetUserFlashMemoryFont_Number(uint8_t PackageIndex, uint8_t CharacterAlignMode);
  uint8_t DisplayImagePackageImage(uint16_t x, uint16_t y, uint8_t packageindex, uint16_t itemindex);
  uint8_t DisplaySDCardImage(uint16_t x, uint16_t y, unsigned char *filename);
  uint8_t DisplaySDCardImage_F(uint16_t x, uint16_t y, const char *filename);

#if DISPLAY_USE_TOUCHPANEL == true
  uint8_t TouchPanel_SetConfig(uint8_t config);
  uint8_t TouchPanel_ReadEvent(void);
  uint8_t TouchPanel_Calibrate(void);
  uint8_t TouchPanel_ReadCont(void);
  uint8_t TouchPanel_ReadDown(void);
  uint8_t TouchPanel_ReadUp(void);
#endif
	
		
  private:
  uint8_t Command(uint16_t command, display_process process);	
  void Init_Hardware();
};


	
#define DELAY_BEFORECMD        // nop(); nop();
#define DELAY_AFTERCMD         // nop(); nop();
#define DELAY_BEFOREDEASSERT   delayMicroseconds(2);

/****
 System Commands
****/

#define CMD_SetBacklightIntensity       0x0003
#define CMD_SetDisplayMode              0x0020  // Landscape=1, Portrait=0
#define CMD_SoftwareReset               0x0030
#define CMD_SoftwareReset_A             0x0232  // Subcommand
#define CMD_DebugMode                   0x0031
#define CMD_SetDisplayMode              0x0020
#define CMD_SetPenColor                 0x0001
#define CMD_SetBrushColor               0x0002

/****
 Numerical Commands
****/
#define CMD_DisplayDecimal8  		    0x0100
#define CMD_DisplayDecimal16 		    0x0101
#define CMD_DisplayDecimal32     	    0x0102

#define CMD_DisplayDecimalSigned8  	    0x0110
#define CMD_DisplayDecimalSigned16 	    0x0111
#define CMD_DisplayDecimalSigned32      0x0112

#define CMD_DisplayHex8				    0x0120
#define CMD_DisplayHex16 			    0x0121
#define CMD_DisplayHex32   			    0x0122
#define CMD_DisplayHex64		  	    0x0123

#define CMD_DisplayBinary8			    0x0130
#define CMD_DisplayBinary16			    0x0131

#define CMD_NumericalConfig             0x0140

/****
 Graphical Commands
****/

#define CMD_FillScreen_BrushColor       0x0021
#define CMD_FillScreen_PenColor         0x0022
#define CMD_FillScreen_RGBColor         0x0023

#define CMD_DrawRect_PenColor           0x0200
#define CMD_DrawRect_PenColorFill       0x0201
#define CMD_DrawRect_BrushColor         0x0202
#define CMD_DrawRect_BrushColorFill     0x0203
#define CMD_DrawRect_RGBColor           0x0204
#define CMD_DrawRect_RGBColorFill       0x0205

#define CMD_DrawCircle_PenColor         0x0210
#define CMD_DrawCircle_BrushColor       0x0211
#define CMD_DrawCircle_RGBColor         0x0212

#define CMD_DrawLine_PenColor           0x0220
#define CMD_DrawLine_BrushColor         0x0221
#define CMD_DrawLine_RGBColor           0x0222

#define CMD_DrawPixel_PenColor          0x0230
#define CMD_DrawPixel_BrushColor        0x0231
#define CMD_DrawPixel_RGBColor          0x0232

#define CMD_MoveTo                      0x0240
#define CMD_LineTo_PenColor             0x0241
#define CMD_LineTo_BrushColor           0x0242
#define CMD_LineTo_RGBColor             0x0243

#define CMD_DrawImage_Begin             0x0250

/****
 Text Commands
****/

#define CMD_DisplayText			        0x0010
#define CMD_SetFont                     0x0011
#define CMD_GetTextWidth         	    0x0012  
#define CMD_SetTextMarginX              0x0013
#define CMD_SetTextFont                 0x0014
#define CMD_SetNumberFont               0x0015

#define CMD_SetUserFlashMemoryFont      0x0016    // from revision 1.3 2428T/60D
#define CMD_SetUserFlashMemoryFont_Text 0x0017    // from revision 1.3 2428T/60D
#define CMD_SetUserFlashMemoryFont_Number 0x0018  // from revision 1.3 2428T/60D

/****
 User Flash Memory
****/

#define CMD_DisplayImagePackage_Image         0xA000
#define CMD_DisplaySDCard_Image               0xA00A

/****
 Touch Panel
****/

#define CMD_TouchPanel_SetConfig              0xA400
#define CMD_TouchPanel_ReadEvent              0xA401
#define CMD_TouchPanel_Calibrate              0xA402

#define CMD_TouchPanel_ReadCont               0xA403
#define CMD_TouchPanel_ReadUp                 0xA404
#define CMD_TouchPanel_ReadDown               0xA405

// Touch Panel Event Flags
#define TOUCH_EVENT_CONT  (1<<2)
#define TOUCH_EVENT_DOWN  (1<<0)
#define TOUCH_EVENT_UP    (1<<1)

// Numerical Display Configuration
#define NCONF_NoPlus     (1<<0)  // Bit Mask display no plus (DisplayDecimalSigned)
#define NCONF_NoZero     (1<<1)  // Bit Mask display no leading zero (DisplayDecimal, DisplayDecimalSigned)
#define NCONF_LeftAdjust (1<<2)  // Bit Mask display left adjust (DisplayDecimal, DisplayDecimalSigned)

// Font Index (embedded font)
#define MAX_EmbeddedFontCount 17
#define INIT_EmbeddedFontIndex 1

// Display Mode
#define dmPortrait  0
#define dmLandscape 1

/* Character Align Mode
   CharacterAlign = 0: proportional character align
   CharacterAlign = 1: left align, constant width
   CharacterAlign = 2: center align, constant width
   CharacterAlign = 3: right align, constant width
*/
#define caProp    0
#define caLeft    1
#define caCenter  2
#define caRight   3

#define nop() __asm volatile ("nop")

/****
 Color definitions 
****/

  #define clBlack  0x000000
  #define clMaroon  0x000080
  #define clGreen  0x008000
  #define clOlive  0x008080
  #define clNavy  0x800000
  #define clPurple  0x800080
  #define clTeal  0x808000
  #define clGray  0x808080
  #define clSilver  0xC0C0C0
  #define clRed  0x0000FF
  #define clLime  0x00FF00
  #define clYellow  0x00FFFF
  #define clBlue  0xFF0000
  #define clFuchsia  0xFF00FF
  #define clAqua  0xFFFF00
  #define clLtGray  0xC0C0C0
  #define clDkGray  0x808080
  #define clWhite  0xFFFFFF
  #define clMoneyGreen  0xC0DCC0
  #define clSkyBlue  0xF0CAA6
  #define clCream  0xF0FBFF
  #define clMedGray  0xA4A0A)


/****
 Color definitions extended
 
  The following "#define cl" values come from the Web Named Color palette and
  are stored in the Windows COLORREF byte order x00bbggrr 
****/


#define clWebSnow  0xFAFAFF
#define clWebFloralWhite  0xF0FAFF
#define clWebLavenderBlush  0xF5F0FF
#define clWebOldLace  0xE6F5FD
#define clWebIvory  0xF0FFFF
#define clWebCornSilk  0xDCF8FF
#define clWebBeige  0xDCF5F5
#define clWebAntiqueWhite  0xD7EBFA
#define clWebWheat  0xB3DEF5
#define clWebAliceBlue  0xFFF8F0
#define clWebGhostWhite  0xFFF8F8
#define clWebLavender  0xFAE6E6
#define clWebSeashell  0xEEF5FF
#define clWebLightYellow  0xE0FFFF
#define clWebPapayaWhip  0xD5EFFF
#define clWebNavajoWhite  0xADDEFF
#define clWebMoccasin  0xB5E4FF
#define clWebBurlywood  0x87B8DE
#define clWebAzure  0xFFFFF0
#define clWebMintcream  0xFAFFF5
#define clWebHoneydew  0xF0FFF0
#define clWebLinen  0xE6F0FA
#define clWebLemonChiffon  0xCDFAFF
#define clWebBlanchedAlmond  0xCDEBFF
#define clWebBisque  0xC4E4FF
#define clWebPeachPuff  0xB9DAFF
#define clWebTan  0x8CB4D2
// yellows/reds yellow -> rosybrown
#define clWebYellow  0x00FFFF
#define clWebDarkOrange  0x008CFF
#define clWebRed  0x0000FF
#define clWebDarkRed  0x00008B
#define clWebMaroon  0x000080
#define clWebIndianRed  0x5C5CCD
#define clWebSalmon  0x7280FA
#define clWebCoral  0x507FFF
#define clWebGold  0x00D7FF
#define clWebTomato  0x4763FF
#define clWebCrimson  0x3C14DC
#define clWebBrown  0x2A2AA5
#define clWebChocolate  0x1E69D2
#define clWebSandyBrown  0x60A4F4
#define clWebLightSalmon  0x7AA0FF
#define clWebLightCoral  0x8080F0
#define clWebOrange  0x00A5FF
#define clWebOrangeRed  0x0045FF
#define clWebFirebrick  0x2222B2
#define clWebSaddleBrown  0x13458B
#define clWebSienna  0x2D52A0
#define clWebPeru  0x3F85CD
#define clWebDarkSalmon  0x7A96E9
#define clWebRosyBrown  0x8F8FBC
// greens palegoldenrod -> darkseagreen
#define clWebPaleGoldenrod  0xAAE8EE
#define clWebLightGoldenrodYellow  0xD2FAFA
#define clWebOlive  0x008080
#define clWebForestGreen  0x228B22
#define clWebGreenYellow  0x2FFFAD
#define clWebChartreuse  0x00FF7F
#define clWebLightGreen  0x90EE90
#define clWebAquamarine  0xD4FF7F
#define clWebSeaGreen  0x578B2E
#define clWebGoldenRod  0x20A5DA
#define clWebKhaki  0x8CE6F0
#define clWebOliveDrab  0x238E6B
#define clWebGreen  0x008000
#define clWebYellowGreen  0x32CD9A
#define clWebLawnGreen  0x00FC7C
#define clWebPaleGreen  0x98FB98
#define clWebMediumAquamarine  0xAACD66
#define clWebMediumSeaGreen  0x71B33C
#define clWebDarkGoldenRod  0x0B86B8
#define clWebDarkKhaki  0x6BB7BD
#define clWebDarkOliveGreen  0x2F6B55
#define clWebDarkgreen  0x006400
#define clWebLimeGreen  0x32CD32
#define clWebLime  0x00FF00
#define clWebSpringGreen  0x7FFF00
#define clWebMediumSpringGreen  0x9AFA00
#define clWebDarkSeaGreen  0x8FBC8F
// greens/blues lightseagreen -> navy
#define clWebLightSeaGreen  0xAAB220
#define clWebPaleTurquoise  0xEEEEAF
#define clWebLightCyan  0xFFFFE0
#define clWebLightBlue  0xE6D8AD
#define clWebLightSkyBlue  0xFACE87
#define clWebCornFlowerBlue  0xED9564
#define clWebDarkBlue  0x8B0000
#define clWebIndigo  0x82004B
#define clWebMediumTurquoise  0xCCD148
#define clWebTurquoise  0xD0E040
#define clWebCyan  0xFFFF00 //   #define clWebAqua
#define clWebAqua  0xFFFF00
#define clWebPowderBlue  0xE6E0B0
#define clWebSkyBlue  0xEBCE87
#define clWebRoyalBlue  0xE16941
#define clWebMediumBlue  0xCD0000
#define clWebMidnightBlue  0x701919
#define clWebDarkTurquoise  0xD1CE00
#define clWebCadetBlue  0xA09E5F
#define clWebDarkCyan  0x8B8B00
#define clWebTeal  0x808000
#define clWebDeepskyBlue  0xFFBF00
#define clWebDodgerBlue  0xFF901E
#define clWebBlue  0xFF0000
#define clWebNavy  0x800000
// violets/pinks darkviolet -> pink
#define clWebDarkViolet  0xD30094
#define clWebDarkOrchid  0xCC3299
#define clWebMagenta  0xFF00FF //   #define clWebFuchsia
#define clWebFuchsia  0xFF00FF
#define clWebDarkMagenta  0x8B008B
#define clWebMediumVioletRed  0x8515C7
#define clWebPaleVioletRed  0x9370DB
#define clWebBlueViolet  0xE22B8A
#define clWebMediumOrchid  0xD355BA
#define clWebMediumPurple  0xDB7093
#define clWebPurple  0x800080
#define clWebDeepPink  0x9314FF
#define clWebLightPink  0xC1B6FF
#define clWebViolet  0xEE82EE
#define clWebOrchid  0xD670DA
#define clWebPlum  0xDDA0DD
#define clWebThistle  0xD8BFD8
#define clWebHotPink  0xB469FF
#define clWebPink  0xCBC0FF
// blue/gray/black lightsteelblue -> black
#define clWebLightSteelBlue  0xDEC4B0
#define clWebMediumSlateBlue  0xEE687B
#define clWebLightSlateGray  0x998877
#define clWebWhite  0xFFFFFF
#define clWebLightgrey  0xD3D3D3
#define clWebGray  0x808080
#define clWebSteelBlue  0xB48246
#define clWebSlateBlue  0xCD5A6A
#define clWebSlateGray  0x908070
#define clWebWhiteSmoke  0xF5F5F5
#define clWebSilver  0xC0C0C0
#define clWebDimGray  0x696969
#define clWebMistyRose  0xE1E4FF
#define clWebDarkSlateBlue  0x8B3D48
#define clWebDarkSlategray  0x4F4F2F
#define clWebGainsboro  0xDCDCDC
#define clWebDarkGray  0xA9A9A9
#define clWebBlack  0x000000
	
	
#endif
